% EX_RIDGE_4D  Example of the ridge regression in (y,x1,x2,x3) coordinates.
%
% Usage:
%   ex_rss_surf; % Random data set will be generated and used
%   ex_rss_surf(csv_file_name); % Name of CSV file with input data
% 
% See also RIDGE_SCALED
function ex_ridge_4d(csvfilename)
close all;
% Work with initial set of point
if nargin == 0
    % Create initial set of data (with multicollinearity between x1 and x3)
    npoints = 100;
    xv = nan(npoints, 3);
    xv(:,1) = 5*rand(npoints,1);
    xv(:,2) = 0.7*(5*rand(npoints,1)) + 0.3*xv(:,1);
    xv(:,3) = 0.1*(5*rand(npoints,1)) + 0.9*xv(:,1);
    yv = 3 + 4*xv(:,1) + 5*xv(:,2) + 6*xv(:,3) + 8*randn(size(xv(:,1)));
else
    % Load inital data set of points from csv file
    data = csvread(csvfilename);
    xv = data(:,1:3);
    yv = data(:,4);
end
% Save current data
csvwrite('ridgedata_4d.csv', [xv yv]);
fd = fopen('ridgedata_4d_comma.csv', 'w');
str = sprintf('%f %f %f %f\n', [xv yv]');
fprintf(fd, '%s', strrep(str, '.', ','));
fclose(fd);
% Show the scatter plots for (xi,xj)
for i = 1:3
    for j = 1:3
        if i ~= j
            subplot(3,3,(i-1)*3 + j);
            plot(xv(:,i),xv(:,j),'b.', 'LineWidth', 2);
            xlabel(['x_', num2str(i)]);
            ylabel(['x_', num2str(j)]);
        end
    end
end
% Perform ridge regression with different values
% of the regularization parameter
kv = 0:0.01:1;
bv = nan(3,numel(kv));
VIFv = nan(3,numel(kv));
for i = 1:numel(kv);
    k = kv(i);
    [b, bst, ~, ~, Cnorm] = ridge_scaled(yv,xv,k);
    fprintf('lambda = %.4f; ', k);
    fprintf('b = ['); fprintf('%.3f ', b); fprintf(']; ');
    fprintf('bst = ['); fprintf('%.4f ', bst); fprintf(']\n');    
    bv(:,i) = bst;
    VIFv(:,i) = diag(Cnorm);   
end

% Draw the results of the regression
% a) Ridge trace plot
figure;
hold on;
plot(kv, bv(1,:), 'k-', 'LineWidth', 2);
plot(kv, bv(2,:), 'r-', 'LineWidth', 2);
plot(kv, bv(3,:), 'b-', 'LineWidth', 2);
xlabel('\lambda', 'FontSize', 16);
ylabel('\beta_i^{*}', 'FontSize', 16);
legend('\beta_1^{*}', '\beta_2^{*}', '\beta_3^{*}');
set(gca, 'FontSize', 14);
box on;
hold off;
% b) VIF (Variance Inflation Factor) plot
figure;
semilogy(kv, VIFv(1,:), 'k-','LineWidth', 2);
hold on;
semilogy(kv, VIFv(2,:), 'r-', 'LineWidth', 2);
semilogy(kv, VIFv(3,:), 'b-', 'LineWidth', 2);
xlabel('\lambda', 'FontSize', 16);
ylabel('VIF_i', 'FontSize', 16);
legend('VIF_1', 'VIF_2', 'VIF_3');
set(gca, 'FontSize', 14);
box on;
hold off;
end
